==> open-xchange-documents-ui_7.8.2.orig.tar.bz2: open-xchange-documents-ui-7.8.2/ui/apps/io.ox/office/spreadsheet/view/render/autostylecache.js <==
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
  * © 2016 OX Software GmbH, Germany. info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/spreadsheet/view/render/autostylecache', [
    'io.ox/office/tk/utils',
    'io.ox/office/tk/utils/simplemap',
    'io.ox/office/tk/object/baseobject',
    'io.ox/office/tk/object/timermixin',
    'io.ox/office/baseframework/app/appobjectmixin',
    'io.ox/office/editframework/utils/color',
    'io.ox/office/spreadsheet/utils/sheetutils',
    'io.ox/office/spreadsheet/utils/paneutils',
    'io.ox/office/spreadsheet/view/render/renderutils'
], function (Utils, SimpleMap, BaseObject, TimerMixin, AppObjectMixin, Color, SheetUtils, PaneUtils, RenderUtils) {

    'use strict';

    // convenience shortcuts
    var StyleDescriptor = RenderUtils.StyleDescriptor;

    // class AutoStyleCache ===================================================

    /**
     * A cache that stores rendering information for all known auto-styles in
     * the document.
     *
     * @constructor
     *
     * @extends BaseObject
     * @extends TimerMixin
     *
     * @param {SpreadsheetView} docView
     *  The document view that contains this cache instance.
     */
    function AutoStyleCache(docView) {

        // self reference
        var self = this;

        // the spreadsheet model
        var docModel = docView.getDocModel();

        // the number formatter of the spreadsheet document
        var numberFormatter = docModel.getNumberFormatter();

        // the cell auto-style collection of the document
        var autoStyles = docModel.getCellAutoStyles();

        // map of all style descriptor caches (separate caches per zoom factor, and grid color)
        var styleCacheMap = new SimpleMap();

        // the current style descriptor cache (according to grid color and zoom)
        var styleCache = null;

        // the current grid line color
        var gridColor = new Color('auto');

        // the current zoom factor
        var sheetZoom = 0;

        // base constructors --------------------------------------------------

        BaseObject.call(this);
        TimerMixin.call(this);
        AppObjectMixin.call(this, docModel.getApp());

        // private methods ----------------------------------------------------

        /**
         * In debug mode, collects the use counts of multiple cache accesses,
         * and prints a message to the browser console.
         */
        var logUsage = RenderUtils.isLoggingActive() ? (function () {

            var counts = { total: 0, hit: 0, update: 0, miss: 0 };

            function collect(type) {
                counts[type] += 1;
                counts.total += 1;
            }

            function format(key) {
                var count = counts[key];
                return (count > 0) ? (', ' + key + '=' + count + ' (' + Math.round(count / counts.total * 1000) / 10 + '%)') : '';
            }

            function flush() {
                RenderUtils.info('AutoStyleCache access statistics: total=' + counts.total + format('hit') + format('update') + format('miss'));
                _.each(counts, function (count, key) { counts[key] = 0; });
            }

            return self.createDebouncedMethod(collect, flush);
        }()) : _.noop;

        /**
         * Returns whether the passed explicit attribute set contains the text
         * color attribute.
         */
        function hasExplicitTextColor(attributeSet) {
            return _.isObject(attributeSet.character) && ('color' in attributeSet.character);
        }

        /**
         * Updates the passed style descriptor, if its own age is less than the
         * age of the current style cache.
         *
         * @param {StyleDescriptor} styleDesc
         *  The style descriptor to be updated if needed.
         *
         * @returns {StyleDescriptor}
         *  The passed style descriptor, for convenience.
         */
        function updateStyleDescriptor(styleDesc, type) {

            // update the style descriptor, if its age is less than the age of this cache
            if (styleDesc.age < styleCache.age) {
                styleDesc.update(docModel, gridColor, sheetZoom);
                styleDesc.age = styleCache.age;
                if (!type) { type = 'update'; }
            }

            logUsage(type || 'hit');
            return styleDesc;
        }

        /**
         * Creates a new style descriptor for the formatting attributes of the
         * specified column, row, or cell descriptor.
         *
         * @param {String} styleId
         *  The identifier of an auto-style.
         *
         * @param {String|Null} borderStyleId
         *  The identifier of an additional auto-style that will be used to
         *  resolve the style settings of all outer border attributes; or null
         *  to use the border attributes of the real auto-style.
         *
         * @param {Object|Null} attributeSet
         *  A non-empty incomplete attribute set with formatting attributes to
         *  be merged over the attributes of the auto-style; or null, if no
         *  explicit attributes will be merged over the auto-style.
         *
         * @returns {StyleDescriptor}
         *  The new style descriptor for the passed column, row, or cell.
         */
        function createStyleDescriptor(styleId, borderStyleId, attributeSet) {

            // the merged attribute set of the auto-style
            var mergedAttributeSet = autoStyles.getMergedAttributeSet(styleId);
            // whether the number format of the auto-style has been overridden by explicit attributes
            var formatChanged = false;
            // whether the text color has been set by an explicit formatting attribute
            var explicitTextColor = false;

            // merge the outer borders of another auto-style
            if (_.isString(borderStyleId)) {
                var borderAttributeSet = autoStyles.getMergedAttributeSet(borderStyleId);
                mergedAttributeSet = _.copy(mergedAttributeSet, true);
                _.each('tblr', function (borderKey) {
                    var borderName = SheetUtils.getBorderName(borderKey);
                    mergedAttributeSet.cell[borderName] = borderAttributeSet.cell[borderName];
                });
            }

            // merge the passed explicit attributes over the resulting attributes
            if (attributeSet) {

                // special handling for number formats: detect whether the format has been changed by the explicit attributes
                var oldParsedFormat = autoStyles.getParsedFormat(styleId);
                mergedAttributeSet = docModel.extendAttributes(mergedAttributeSet, attributeSet, { clone: true });
                var newParsedFormat = numberFormatter.getParsedFormatForAttributes(mergedAttributeSet.cell);
                formatChanged = oldParsedFormat !== newParsedFormat;

                // detect whether the text color has been set explicitly
                explicitTextColor = hasExplicitTextColor(attributeSet);
            }

            // if the text color has not been set explicitly by the passed attributes, it may be part of the auto-style itself
            if (!explicitTextColor) {
                explicitTextColor = hasExplicitTextColor(autoStyles.getExplicitAttributeSet(styleId));
            }

            // create a new style descriptor
            var styleDesc = new StyleDescriptor(mergedAttributeSet, formatChanged, explicitTextColor);
            styleDesc.age = 0;

            // update the formatting information of the style descriptor
            return updateStyleDescriptor(styleDesc, 'miss');
        }

        function storeStyleKey(styleId, styleKey) {
            var styleKeys = styleCache.keys[styleId] || (styleCache.keys[styleId] = []);
            styleKeys.push(styleKey);
        }

        function deleteCachedStyles(styleId) {
            styleCacheMap.forEach(function (cacheEntry) {
                var styleKeys = cacheEntry.keys[styleId];
                if (styleKeys) {
                    styleKeys.forEach(function (styleKey) {
                        delete cacheEntry.map[styleKey];
                    });
                }
            });
        }

        // public methods -----------------------------------------------------

        /**
         * Returns a style descriptor with rendering information for the
         * specified cell auto-style.
         *
         * @param {String} styleId
         *  The identifier of an auto-style.
         *
         * @param {Object} [options]
         *  Optional parameters:
         *  @param {String} [options.borderStyleId]
         *      The identifier of an additional auto-style that will be used to
         *      resolve the style settings of all outer border attributes.
         *  @param {Object} [options.attributeSet]
         *      An incomplete attribute set with formatting attributes to be
         *      merged over the attributes of the auto-style. MUST NOT contain
         *      a style sheet identifier.
         *
         * @returns {StyleDescriptor}
         *  The style descriptor for the specified auto-style.
         */
        this.getStyle = function (styleId, options) {

            // normalize the default auto-style identifier
            styleId = autoStyles.getEffectiveStyleId(styleId);

            // an optional auto-style identifier for outer border attributes
            var borderStyleId = Utils.getStringOption(options, 'borderStyleId', null);
            if (autoStyles.areEqualStyleIds(styleId, borderStyleId)) { borderStyleId = null; }

            // an optional attribute set to be merged over the auto-style
            var attributeSet = Utils.getObjectOption(options, 'attributeSet', null);
            if (_.isEmpty(attributeSet)) { attributeSet = null; }

            // the cache key for the auto-style, and the explicit attributes
            var styleKey = Utils.stringifyJSON([styleId, borderStyleId, attributeSet]);

            // if style descriptor exists already in the cache, update it on demand
            var styleDesc = styleCache.map[styleKey];
            if (styleDesc) { return updateStyleDescriptor(styleDesc); }

            // cache the complete style keys separated by auto-style identifiers
            // (needed to clean cache entries for changed or deleted auto-styles)
            storeStyleKey(styleId, styleKey);
            if (_.isString(borderStyleId)) { storeStyleKey(borderStyleId, styleKey); }

            // create a new style descriptor for an auto-style, and store it in the cache
            return (styleCache.map[styleKey] = createStyleDescriptor(styleId, borderStyleId, attributeSet));
        };

        /**
         * Updates this cache according to the grid color and zoom index of the
         * active sheet in the spreadsheet document, and marks the cached style
         * descriptors to be dirty, if the grid color or zoom factor has really
         * been changed (the current grid color is used to render cell border
         * lines with automatic line color). When accessing a dirty style
         * descriptor the next time with the method AutoStyleCache.getStyle(),
         * its formatting settings will be updated automatically.
         *
         * @returns {AutoStyleCache}
         *  A reference to this instance.
         */
        this.updateStyles = function () {

            // model of the active sheet in the document
            var sheetModel = docView.getSheetModel();
            // current grid color of the active sheet
            var newGridColor = sheetModel.getGridColor();
            // current grid color, as CSS color value
            var oldCssColor = docModel.resolveColor(gridColor, 'line').css;
            // new grid color, as CSS color value
            var newCssColor = docModel.resolveColor(newGridColor, 'line').css;
            // whether the grid color has been changed
            var gridColorChanged = oldCssColor !== newCssColor;
            // current zoom factor of the active sheet
            var newSheetZoom = sheetModel.getEffectiveZoom();
            // whether the zoom factor has been changed
            var sheetZoomChanged = sheetZoom !== newSheetZoom;

            // nothing to do, if neither effective CSS grid color nor zoom factor have changed
            if (!gridColorChanged && !sheetZoomChanged) { return this; }

            // store the new settings, mark the styles to be dirty, but do not touch the descriptors yet
            gridColor = newGridColor;
            sheetZoom = newSheetZoom;

            // use permanent cache for simple grid colors
            var permColorKey = /^#(00|FF){3}$/i.test(newCssColor) ? newCssColor.toUpperCase() : '';
            // use permanent cache for predefined zoom factors
            var permZoomKey = _.contains(PaneUtils.ZOOM_FACTORS, sheetZoom) ? Math.round(sheetZoom * 100) : '';
            // key of the current style descriptor cache
            var cacheKey = permColorKey + ':' + permZoomKey;

            // create the style descriptor cache if missing
            styleCache = styleCacheMap.getOrCreate(cacheKey, function () { return { map: {}, keys: {}, age: 1 }; });

            // update the age of non-permanent style caches, if grid color or zoom factor has changed
            if ((gridColorChanged && !permColorKey) || (sheetZoomChanged && !permZoomKey)) {
                RenderUtils.log('AutoStyleCache.updateStyles(): style cache "' + cacheKey + '" invalidated');
                styleCache.age += 1;
            } else {
                RenderUtils.log('AutoStyleCache.updateStyles(): style cache "' + cacheKey + '" activated');
            }

            return this;
        };

        // initialization -----------------------------------------------------

        // invalidate the cache after inserting new fonts (CSS font family chains may change)
        this.listenTo(docModel.getFontCollection(), 'triggered', function () {
            styleCacheMap.clear();
            self.updateStyles();
        });

        // remove all cached data of changed and deleted auto-styles
        this.listenTo(autoStyles, 'change:autostyle delete:autostyle', function (event, styleId) {
            deleteCachedStyles(styleId);
        });

        // destroy all class members on destruction
        this.registerDestructor(function () {
            self = docView = docModel = numberFormatter = null;
            autoStyles = styleCacheMap = styleCache = null;
        });

    } // class AutoStyleCache

    // exports ================================================================

    return BaseObject.extend({ constructor: AutoStyleCache });

});

==> open-xchange-documents-ui_7.8.3.orig.tar.bz2: open-xchange-documents-ui-7.8.3/ui/apps/io.ox/office/spreadsheet/view/render/autostylecache.js <==
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) 2016 OX Software GmbH
 * Mail: info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define('io.ox/office/spreadsheet/view/render/autostylecache', [
    'io.ox/office/tk/utils',
    'io.ox/office/tk/container/simplemap',
    'io.ox/office/tk/object/baseobject',
    'io.ox/office/tk/object/timermixin',
    'io.ox/office/baseframework/app/appobjectmixin',
    'io.ox/office/editframework/utils/color',
    'io.ox/office/spreadsheet/utils/sheetutils',
    'io.ox/office/spreadsheet/utils/paneutils',
    'io.ox/office/spreadsheet/view/render/renderutils'
], function (Utils, SimpleMap, BaseObject, TimerMixin, AppObjectMixin, Color, SheetUtils, PaneUtils, RenderUtils) {

    'use strict';

    // convenience shortcuts
    var StyleDescriptor = RenderUtils.StyleDescriptor;

    // class AutoStyleCache ===================================================

    /**
     * A cache that stores rendering information for all known auto-styles in
     * the document.
     *
     * @constructor
     *
     * @extends BaseObject
     * @extends TimerMixin
     *
     * @param {SpreadsheetView} docView
     *  The document view that contains this cache instance.
     */
    function AutoStyleCache(docView) {

        // self reference
        var self = this;

        // the spreadsheet model
        var docModel = docView.getDocModel();

        // the number formatter of the spreadsheet document
        var numberFormatter = docModel.getNumberFormatter();

        // the cell auto-style collection of the document
        var autoStyles = docModel.getCellAutoStyles();

        // map of all style descriptor caches (separate caches per zoom factor, and grid color)
        var styleCacheMap = new SimpleMap();

        // the current style descriptor cache (according to grid color and zoom)
        var styleCache = null;

        // the current grid line color
        var gridColor = new Color('auto');

        // the current zoom factor
        var sheetZoom = 0;

        // base constructors --------------------------------------------------

        BaseObject.call(this);
        TimerMixin.call(this);
        AppObjectMixin.call(this, docModel.getApp());

        // private methods ----------------------------------------------------

        /**
         * In debug mode, collects the use counts of multiple cache accesses,
         * and prints a message to the browser console.
         */
        var logUsage = RenderUtils.isLoggingActive() ? (function () {

            var counts = { total: 0, hit: 0, update: 0, miss: 0 };

            function collect(type) {
                counts[type] += 1;
                counts.total += 1;
            }

            function format(key) {
                var count = counts[key];
                return (count > 0) ? (', ' + key + '=' + count + ' (' + Math.round(count / counts.total * 1000) / 10 + '%)') : '';
            }

            function flush() {
                RenderUtils.info('AutoStyleCache access statistics: total=' + counts.total + format('hit') + format('update') + format('miss'));
                _.each(counts, function (count, key) { counts[key] = 0; });
            }

            return self.createDebouncedMethod(collect, flush, { delay: 500, infoString: 'AutoStyleCache: logUsage', app: docView.getApp() });
        }()) : _.noop;

        /**
         * Returns whether the passed explicit attribute set contains the text
         * color attribute.
         */
        function hasExplicitTextColor(attributeSet) {
            return _.isObject(attributeSet.character) && ('color' in attributeSet.character);
        }

        /**
         * Updates the passed style descriptor, if its own age is less than the
         * age of the current style cache.
         *
         * @param {StyleDescriptor} styleDesc
         *  The style descriptor to be updated if needed.
         *
         * @returns {StyleDescriptor}
         *  The passed style descriptor, for convenience.
         */
        function updateStyleDescriptor(styleDesc, type) {

            // update the style descriptor, if its age is less than the age of this cache
            if (styleDesc.age < styleCache.age) {
                styleDesc.update(docModel, gridColor, sheetZoom);
                styleDesc.age = styleCache.age;
                if (!type) { type = 'update'; }
            }

            logUsage(type || 'hit');
            return styleDesc;
        }

        /**
         * Creates a new style descriptor for the formatting attributes of the
         * specified column, row, or cell descriptor.
         *
         * @param {String} styleId
         *  The identifier of an auto-style.
         *
         * @param {String|Null} borderStyleId
         *  The identifier of an additional auto-style that will be used to
         *  resolve the style settings of all outer border attributes; or null
         *  to use the border attributes of the real auto-style.
         *
         * @param {Object|Null} attributeSet
         *  A non-empty incomplete attribute set with formatting attributes to
         *  be merged over the attributes of the auto-style; or null, if no
         *  explicit attributes will be merged over the auto-style.
         *
         * @returns {StyleDescriptor}
         *  The new style descriptor for the passed column, row, or cell.
         */
        function createStyleDescriptor(styleId, borderStyleId, attributeSet) {

            // the merged attribute set of the auto-style
            var mergedAttributeSet = autoStyles.getMergedAttributeSet(styleId);
            // whether the number format of the auto-style has been overridden by explicit attributes
            var formatChanged = false;
            // whether the text color has been set by an explicit formatting attribute
            var explicitTextColor = false;

            // merge the outer borders of another auto-style
            if (_.isString(borderStyleId)) {
                var borderAttributeSet = autoStyles.getMergedAttributeSet(borderStyleId);
                mergedAttributeSet = _.copy(mergedAttributeSet, true);
                _.each('tblr', function (borderKey) {
                    var borderName = SheetUtils.getBorderName(borderKey);
                    mergedAttributeSet.cell[borderName] = borderAttributeSet.cell[borderName];
                });
            }

            // merge the passed explicit attributes over the resulting attributes
            if (attributeSet) {

                // special handling for number formats: detect whether the format has been changed by the explicit attributes
                var oldParsedFormat = autoStyles.getParsedFormat(styleId);
                mergedAttributeSet = docModel.extendAttributes(mergedAttributeSet, attributeSet, { clone: true });
                var newParsedFormat = numberFormatter.getParsedFormatForAttributes(mergedAttributeSet.cell);
                formatChanged = oldParsedFormat !== newParsedFormat;

                // detect whether the text color has been set explicitly
                explicitTextColor = hasExplicitTextColor(attributeSet);
            }

            // if the text color has not been set explicitly by the passed attributes, it may be part of the auto-style itself
            if (!explicitTextColor) {
                explicitTextColor = hasExplicitTextColor(autoStyles.getExplicitAttributeSet(styleId));
            }

            // create a new style descriptor
            var styleDesc = new StyleDescriptor(mergedAttributeSet, formatChanged, explicitTextColor);
            styleDesc.age = 0;

            // update the formatting information of the style descriptor
            return updateStyleDescriptor(styleDesc, 'miss');
        }

        function storeStyleKey(styleId, styleKey) {
            var styleKeys = styleCache.keys[styleId] || (styleCache.keys[styleId] = []);
            styleKeys.push(styleKey);
        }

        function deleteCachedStyles(styleId) {
            styleCacheMap.forEach(function (cacheEntry) {
                var styleKeys = cacheEntry.keys[styleId];
                if (styleKeys) {
                    styleKeys.forEach(function (styleKey) {
                        delete cacheEntry.map[styleKey];
                    });
                }
            });
        }

        // public methods -----------------------------------------------------

        /**
         * Returns a style descriptor with rendering information for the
         * specified cell auto-style.
         *
         * @param {String} styleId
         *  The identifier of an auto-style.
         *
         * @param {Object} [options]
         *  Optional parameters:
         *  @param {String} [options.borderStyleId]
         *      The identifier of an additional auto-style that will be used to
         *      resolve the style settings of all outer border attributes.
         *  @param {Object} [options.attributeSet]
         *      An incomplete attribute set with formatting attributes to be
         *      merged over the attributes of the auto-style. MUST NOT contain
         *      a style sheet identifier.
         *
         * @returns {StyleDescriptor}
         *  The style descriptor for the specified auto-style.
         */
        this.getStyle = function (styleId, options) {

            // normalize the default auto-style identifier
            styleId = autoStyles.getEffectiveStyleId(styleId);

            // an optional auto-style identifier for outer border attributes
            var borderStyleId = Utils.getStringOption(options, 'borderStyleId', null);
            if (autoStyles.areEqualStyleIds(styleId, borderStyleId)) { borderStyleId = null; }

            // an optional attribute set to be merged over the auto-style
            var attributeSet = Utils.getObjectOption(options, 'attributeSet', null);
            if (_.isEmpty(attributeSet)) { attributeSet = null; }

            // the cache key for the auto-style, and the explicit attributes
            var styleKey = Utils.stringifyJSON([styleId, borderStyleId, attributeSet]);

            // if style descriptor exists already in the cache, update it on demand
            var styleDesc = styleCache.map[styleKey];
            if (styleDesc) { return updateStyleDescriptor(styleDesc); }

            // cache the complete style keys separated by auto-style identifiers
            // (needed to clean cache entries for changed or deleted auto-styles)
            storeStyleKey(styleId, styleKey);
            if (_.isString(borderStyleId)) { storeStyleKey(borderStyleId, styleKey); }

            // create a new style descriptor for an auto-style, and store it in the cache
            return (styleCache.map[styleKey] = createStyleDescriptor(styleId, borderStyleId, attributeSet));
        };

        /**
         * Updates this cache according to the grid color and zoom index of the
         * active sheet in the spreadsheet document, and marks the cached style
         * descriptors to be dirty, if the grid color or zoom factor has really
         * been changed (the current grid color is used to render cell border
         * lines with automatic line color). When accessing a dirty style
         * descriptor the next time with the method AutoStyleCache.getStyle(),
         * its formatting settings will be updated automatically.
         *
         * @returns {AutoStyleCache}
         *  A reference to this instance.
         */
        this.updateStyles = function () {

            // model of the active sheet in the document
            var sheetModel = docView.getSheetModel();
            // current grid color of the active sheet
            var newGridColor = sheetModel.getGridColor();
            // current grid color, as CSS color value
            var oldCssColor = docModel.resolveColor(gridColor, 'line').css;
            // new grid color, as CSS color value
            var newCssColor = docModel.resolveColor(newGridColor, 'line').css;
            // whether the grid color has been changed
            var gridColorChanged = oldCssColor !== newCssColor;
            // current zoom factor of the active sheet
            var newSheetZoom = sheetModel.getEffectiveZoom();
            // whether the zoom factor has been changed
            var sheetZoomChanged = sheetZoom !== newSheetZoom;

            // nothing to do, if neither effective CSS grid color nor zoom factor have changed
            if (!gridColorChanged && !sheetZoomChanged) { return this; }

            // store the new settings, mark the styles to be dirty, but do not touch the descriptors yet
            gridColor = newGridColor;
            sheetZoom = newSheetZoom;

            // use permanent cache for simple grid colors
            var permColorKey = /^#(00|FF){3}$/i.test(newCssColor) ? newCssColor.toUpperCase() : '';
            // use permanent cache for predefined zoom factors
            var permZoomKey = _.contains(PaneUtils.ZOOM_FACTORS, sheetZoom) ? Math.round(sheetZoom * 100) : '';
            // key of the current style descriptor cache
            var cacheKey = permColorKey + ':' + permZoomKey;

            // create the style descriptor cache if missing
            styleCache = styleCacheMap.getOrCreate(cacheKey, function () { return { map: {}, keys: {}, age: 1 }; });

            // update the age of non-permanent style caches, if grid color or zoom factor has changed
            if ((gridColorChanged && !permColorKey) || (sheetZoomChanged && !permZoomKey)) {
                RenderUtils.log('AutoStyleCache.updateStyles(): style cache "' + cacheKey + '" invalidated');
                styleCache.age += 1;
            } else {
                RenderUtils.log('AutoStyleCache.updateStyles(): style cache "' + cacheKey + '" activated');
            }

            return this;
        };

        // initialization -----------------------------------------------------

        // invalidate the cache after inserting new fonts (CSS font family chains may change)
        this.listenTo(docModel.getFontCollection(), 'triggered', function () {
            styleCacheMap.clear();
            self.updateStyles();
        });

        // remove all cached data of changed and deleted auto-styles
        this.listenTo(autoStyles, 'change:autostyle delete:autostyle', function (event, styleId) {
            deleteCachedStyles(styleId);
        });

        // destroy all class members on destruction
        this.registerDestructor(function () {
            self = docView = docModel = numberFormatter = null;
            autoStyles = styleCacheMap = styleCache = null;
        });

    } // class AutoStyleCache

    // exports ================================================================

    return BaseObject.extend({ constructor: AutoStyleCache });

});
